# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.i18n import JavaScriptCatalog
from django.views.static import serve

from .utils import load_from_file

admin.autodiscover()

urlpatterns = [
    url(r"^media/(?P<path>.*)$", serve, {"document_root": settings.MEDIA_ROOT, "show_indexes": True}),  # NOQA
    url(r"^jsi18n/(?P<packages>\S+?)/$", JavaScriptCatalog.as_view()),  # NOQA
]
i18n_urls = [
    url(r"^admin/", admin.site.urls),
]

try:
    load_from_file("%s.urls" % settings.BASE_APPLICATION)
    i18n_urls.append(url(r"^%s/" % settings.BASE_APPLICATION, include("%s.urls" % settings.BASE_APPLICATION)))  # NOQA
except IOError:
    pass

if settings.USE_CMS:
    i18n_urls.append(url(r"^", include("cms.urls")))  # NOQA

urlpatterns += i18n_patterns(*i18n_urls)
urlpatterns += staticfiles_urlpatterns()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import os
import os.path
import shutil
import sys
from copy import copy
from distutils.version import LooseVersion
from tempfile import mkdtemp

import django

from app_helper import runner
from app_helper.default_settings import get_boilerplates_settings
from app_helper.main import _make_settings, _map_argv, core
from app_helper.utils import DJANGO_1_9, captured_output, temp_dir, work_in

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

try:
    import unittest2 as unittest
except ImportError:
    import unittest

DEFAULT_ARGS = {
    "shell": False,
    "test": False,
    "cms_check": False,
    "compilemessages": False,
    "makemessages": False,
    "makemigrations": False,
    "pyflakes": False,
    "authors": False,
    "server": False,
    "--xvfb": "",
    "--runner": None,
    "--runner-options": None,
    "--cms": True,
    "--failfast": False,
    "--merge": False,
    "--locale": "",
    "--boilerplate": False,
    "--dry-run": False,
    "--empty": False,
    "--native": False,
    "--persistent": False,
    "--persistent-path": None,
    "--bind": "",
    "--port": "",
    "<test-label>": "",
    "<extra-applications>": "",
    "options": "",
    "<command>": "",
}


class CommandTests(unittest.TestCase):
    application = None
    basedir = None
    pofile = None
    mofile = None
    migration_dir = None

    @classmethod
    def setUpClass(cls):
        os.environ.setdefault("DATABASE_URL", "sqlite://localhost/:memory:")
        cls.basedir = os.path.abspath(os.path.join("app_helper", "test_utils"))
        cls.application = "example1"
        cls.application_2 = "example2"
        with work_in(cls.basedir):
            with captured_output():
                cls.migration_example = os.path.abspath(
                    os.path.join(cls.application, "data", "django_0001_initial.py")
                )
                cls.migration_partial = os.path.abspath(
                    os.path.join(cls.application, "data", "django_0001_partial.py")
                )
                cls.poexample = os.path.abspath(os.path.join(cls.application, "data", "django.po"))
                cls.pofile = os.path.abspath(os.path.join(cls.application, "locale", "en", "LC_MESSAGES", "django.po"))
                cls.mofile = os.path.abspath(os.path.join(cls.application, "locale", "en", "LC_MESSAGES", "django.mo"))
                cls.migration_dir = os.path.abspath(os.path.join(cls.application, "migrations"))
                cls.migration_dir_2 = os.path.abspath(os.path.join(cls.application_2, "migrations"))
                cls.migration_file = os.path.abspath(os.path.join(cls.application, "migrations", "0001_initial.py"))
                cls.migration_file_2 = os.path.abspath(
                    os.path.join(cls.application_2, "migrations", "0001_initial.py")
                )
        try:
            import cms
        except ImportError:
            DEFAULT_ARGS["--cms"] = False

    def setUp(self):
        try:
            os.unlink(self.pofile)
        except (OSError, TypeError):
            pass
        try:
            os.unlink(self.mofile)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir:
                shutil.rmtree(self.migration_dir)
        except (OSError, TypeError):
            pass
        try:
            if self.migration_dir_2:
                shutil.rmtree(self.migration_dir_2)
        except (OSError, TypeError):
            pass
        try:
            del sys.modules["example1.migrations"]
        except KeyError as e:
            pass
        try:
            del sys.modules["example2.migrations"]
        except KeyError as e:
            pass

    def tearDown(self):
        os.environ["AUTH_USER_MODEL"] = "auth.User"
        self.setUp()

    def test_map_argv(self):
        from ..main import _map_argv

        argv_1 = [
            "helper.py",
            "example1",
            "test",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_1 = {
            "--bind": "127.0.0.1",
            "--boilerplate": False,
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "<application>": "example1",
            "<command>": None,
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "test",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "pyflakes": False,
            "server": False,
            "setup": False,
            "test": True,
        }
        argv_2 = [
            "helper.py",
            "example1",
            "server",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_2 = {
            "--bind": "127.0.0.1",
            "--boilerplate": False,
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "<application>": "example1",
            "<command>": None,
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "server",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "pyflakes": False,
            "server": True,
            "setup": False,
            "test": False,
        }

        argv_3 = [
            "helper.py",
            "example1",
            "some_command",
            "--cms",
            "--persistent-path=test",
            "--extra-settings==something.py",
            "--runner==something.py",
        ]
        target_3 = {
            "--bind": "127.0.0.1",
            "--boilerplate": False,
            "--cms": True,
            "--dry-run": False,
            "--empty": False,
            "--extra-settings": "",
            "--failfast": False,
            "--help": False,
            "--locale": None,
            "--merge": False,
            "--migrate": False,
            "--native": False,
            "--no-migrate": False,
            "--persistent": True,
            "--persistent-path": "test",
            "--port": "8000",
            "--runner": "",
            "--runner-options": None,
            "--verbose": None,
            "--version": False,
            "--xvfb": False,
            "<application>": "example1",
            "<command>": "some_command",
            "<extra-applications>": [],
            "<test-label>": [],
            "authors": False,
            "cms_check": False,
            "compilemessages": False,
            "makemessages": False,
            "makemigrations": False,
            "options": [
                "helper.py",
                "some_command",
                "--cms",
                "--persistent-path=test",
                "--extra-settings==something.py",
                "--runner==something.py",
            ],
            "pyflakes": False,
            "server": False,
            "setup": False,
            "test": False,
        }

        application_module = __import__(argv_1[1])
        args = _map_argv(argv_1, application_module)
        self.assertEqual(target_1, args)

        application_module = __import__(argv_2[1])
        args = _map_argv(argv_2, application_module)
        self.assertEqual(target_2, args)

        application_module = __import__(argv_3[1])
        args = _map_argv(argv_3, application_module)
        self.assertEqual(target_3, args)

    def test_extra_settings(self):
        from django.conf import settings

        with work_in(self.basedir):
            # with captured_output() as (out, err):
            args = copy(DEFAULT_ARGS)
            with temp_dir() as STATIC_ROOT:
                with temp_dir() as MEDIA_ROOT:
                    local_settings = _make_settings(args, self.application, settings, STATIC_ROOT, MEDIA_ROOT)
                    # Testing that helper.py in custom project is loaded
                    self.assertEqual(local_settings.TIME_ZONE, "Europe/Rome")

                    args["--boilerplate"] = True
                    args["--extra-settings"] = "cms_helper_extra.py"
                    local_settings = _make_settings(args, self.application, settings, STATIC_ROOT, MEDIA_ROOT)
                    # Testing that helper.py in the command option is loaded
                    self.assertEqual(local_settings.TIME_ZONE, "Europe/Paris")
                    # Existing application is kept
                    self.assertTrue("app_helper.test_data" in local_settings.INSTALLED_APPS)
                    # New ones are added both on top and in random positions
                    self.assertEqual("djangocms_admin_style", local_settings.INSTALLED_APPS[0])
                    self.assertTrue("some_app" in local_settings.INSTALLED_APPS)

                    # Ditto for middlewares
                    if DJANGO_1_9:
                        self.assertEqual("top_middleware", local_settings.MIDDLEWARE_CLASSES[0])
                        self.assertTrue("some_middleware" in local_settings.MIDDLEWARE_CLASSES)
                        self.assertTrue(
                            "django.contrib.sessions.middleware.SessionMiddleware" in local_settings.MIDDLEWARE_CLASSES
                        )
                    else:
                        self.assertTrue(
                            "django.contrib.sessions.middleware.SessionMiddleware" in local_settings.MIDDLEWARE
                        )
                        self.assertEqual("top_middleware", local_settings.MIDDLEWARE[0])
                        self.assertTrue("some_middleware" in local_settings.MIDDLEWARE)

                    boilerplate_settings = get_boilerplates_settings()

                    # Check the loaders
                    self.assertTrue(
                        "django.template.loaders.app_directories.Loader"
                        in local_settings.TEMPLATES[0]["OPTIONS"]["loaders"]
                    )
                    # Loaders declared in settings
                    self.assertTrue(
                        "admin_tools.template_loaders.Loader" in local_settings.TEMPLATES[0]["OPTIONS"]["loaders"]
                    )
                    # Existing application is kept
                    self.assertTrue(
                        "django.template.context_processors.request"
                        in local_settings.TEMPLATES[0]["OPTIONS"]["context_processors"]
                    )
                    # New one is added
                    self.assertTrue(
                        "django.template.context_processors.debug"
                        in local_settings.TEMPLATES[0]["OPTIONS"]["context_processors"]
                    )
                    # Check template dirs
                    self.assertTrue("some/dir" in local_settings.TEMPLATES[0]["DIRS"])
                    # Check for aldryn boilerplates
                    for name, value in boilerplate_settings.items():
                        if not name.startswith("TEMPLATE"):
                            if type(value) in (list, tuple):
                                self.assertTrue(set(getattr(local_settings, name)).intersection(set(value)))
                            elif name == "ALDRYN_BOILERPLATE_NAME":
                                self.assertEqual(getattr(local_settings, name), "legacy")
                            else:
                                self.assertTrue(value in getattr(local_settings, name))
                        elif name == "TEMPLATE_CONTEXT_PROCESSORS":
                            self.assertTrue(
                                set(local_settings.TEMPLATES[0]["OPTIONS"]["context_processors"]).intersection(
                                    set(value)
                                )
                            )
                        elif name == "TEMPLATE_LOADERS":
                            self.assertTrue(
                                set(local_settings.TEMPLATES[0]["OPTIONS"]["loaders"]).intersection(set(value))
                            )

    @patch("app_helper.main.autoreload")
    def test_server(self, mocked_command):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["server"] = True
                core(args, self.application)
            self.assertTrue("A admin user (username: admin, password: admin) has been created." in out.getvalue())

    def test_makemigrations(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["makemigrations"] = True
                args["<extra-applications>"] = ["example2"]
                core(args, self.application)
            self.assertTrue(os.path.exists(self.migration_file))
            self.assertTrue(os.path.exists(self.migration_file_2))
        self.assertTrue("Create model ExampleModel1" in out.getvalue())
        self.assertTrue("Create model ExampleModel2" in out.getvalue())

    def skip_test_makemigrations_update(self):
        os.makedirs(self.migration_dir)
        open(os.path.join(self.migration_dir, "__init__.py"), "w")
        shutil.copy(self.migration_partial, self.migration_file)
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["makemigrations"] = True
                core(args, self.application)
        self.assertTrue("Migrations for 'example1':" in out.getvalue())

    def test_makemigrations_empty(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                os.makedirs(self.migration_dir)
                open(os.path.join(self.migration_dir, "__init__.py"), "w")
                shutil.copy(self.migration_example, self.migration_file)
                args = copy(DEFAULT_ARGS)
                args["makemigrations"] = True
                args["--empty"] = True
                core(args, self.application)
        self.assertTrue("Migrations for 'example1':" in out.getvalue())

    def test_makemigrations_merge(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["makemigrations"] = True
                args["--merge"] = True
                core(args, self.application)
                self.assertTrue("No conflicts detected to merge" in out.getvalue())

    def test_makemessages(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                core(args, self.application)
                self.assertTrue(os.path.exists(self.pofile))

    def test_compilemessages(self):
        with work_in(self.basedir):
            shutil.copy(self.poexample, self.pofile)
            args = copy(DEFAULT_ARGS)
            args["compilemessages"] = True
            core(args, self.application)
            self.assertTrue(os.path.exists(self.mofile))

    def test_cms_check(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["cms_check"] = True
                args["--extra-settings"] = "helper.py"
                args["--migrate"] = False
                core(args, self.application)
            self.assertTrue("Installation okay" in out.getvalue())
            self.assertFalse("[WARNING]" in out.getvalue())
            self.assertFalse("[ERROR]" in out.getvalue())

    @unittest.skipIf(LooseVersion(django.get_version()) >= LooseVersion("1.9"), reason="Test for Django up until 1.8")
    def test_cms_check_nocms(self):
        try:
            import cms

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["cms_check"] = True
                args["--extra-settings"] = "helper.py"
                args["--migrate"] = False
                core(args, self.application)
            self.assertTrue("cms_check available only if django CMS is installed" in out.getvalue())

    @unittest.skipIf(LooseVersion(django.get_version()) < LooseVersion("1.9"), reason="Test for Django 1.9+")
    def test_cms_check_nocms_19(self):
        try:
            import cms

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        with work_in(self.basedir):
            with self.assertRaises(ImportError):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["cms_check"] = True
                args["--extra-settings"] = "helper.py"
                args["--migrate"] = False
                core(args, self.application)

    @unittest.skipIf(
        LooseVersion(django.get_version()) < LooseVersion("1.7"), reason="check command available in Django 1.7+ only"
    )
    def test_any_command_check(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "check"
                args["options"] = ["helper", "check", "--extra-settings=cms_helper_extra.py"]
                core(args, self.application)
        self.assertTrue("no issues" in out.getvalue())

    def test_any_command_compilemessages(self):
        with work_in(os.path.join(self.basedir, self.application)):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "compilemessages"
                args["options"] = "helper compilemessages --cms -len --verbosity=2".split(" ")
                core(args, self.application)
                self.assertTrue(os.path.exists(self.mofile))

    def test_any_command_migrations(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "makemigrations"
                args["options"] = "helper makemigrations example2 --verbosity=2".split(" ")
                core(args, self.application)
            self.assertFalse("Create model ExampleModel1" in out.getvalue())
            self.assertFalse(os.path.exists(self.migration_file))
            self.assertTrue("Create model ExampleModel2" in out.getvalue())
            self.assertTrue(os.path.exists(self.migration_file_2))

    @unittest.skipIf(sys.version_info[:2] in ((3, 8),),
                     reason='This test fails on Python 3.8+')
    def test_pyflakes(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["pyflakes"] = True
                core(args, self.application)
        self.assertFalse(os.path.exists(args["STATIC_ROOT"]))
        self.assertFalse(os.path.exists(args["MEDIA_ROOT"]))

    def test_pyflakes_nocms(self):
        try:
            import cms

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["pyflakes"] = True
                core(args, self.application)
            self.assertTrue(
                "Static analysis available only if django CMS and pyflakes are installed" in out.getvalue()
            )
        self.assertFalse(os.path.exists(args["STATIC_ROOT"]))
        self.assertFalse(os.path.exists(args["MEDIA_ROOT"]))

    def test_testrun(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        path = os.path.join(self.basedir, "data")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState

                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["--persistent"] = True
                    args["--runner"] = "runners.CapturedOutputRunner"
                    core(args, self.application)
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)
        self.assertTrue(args["STATIC_ROOT"].startswith(path))
        self.assertTrue(args["MEDIA_ROOT"].startswith(path))
        self.assertTrue(os.path.exists(args["STATIC_ROOT"]))
        self.assertTrue(os.path.exists(args["MEDIA_ROOT"]))

    def test_testrun_persistent_path(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        path = mkdtemp()
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState

                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["--persistent"] = True
                    args["--persistent-path"] = path
                    args["--runner"] = "runners.CapturedOutputRunner"
                    core(args, self.application)
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)
        self.assertTrue(args["STATIC_ROOT"].startswith(path))
        self.assertTrue(args["MEDIA_ROOT"].startswith(path))
        self.assertTrue(os.path.exists(args["STATIC_ROOT"]))
        self.assertTrue(os.path.exists(args["MEDIA_ROOT"]))

    def test_runner_wrong(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            if sys.version_info < (3, 5):
                exception = ImportError
            else:
                exception = SystemExit
            with captured_output() as (out, err):
                with self.assertRaises(exception) as exit:
                    args = list()
                    args.append("helper")
                    args.append("test")
                    args.append("example1")
                    args.append("--runner=runners.CapturedOutputRunner")
                    args.append("whatever")
                    runner.cms("example1", args)
        if sys.version_info >= (3, 5):
            self.assertEqual(exit.exception.code, 1)

    def test_runner(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        from app_helper.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit:
                        args = list()
                        args.append("helper")
                        args.append("test")
                        args.append("--extra-settings=helper.py")
                        args.append("example1")
                        runner.cms("example1", args)
        self.assertTrue("visible string" in out.getvalue())
        self.assertFalse("hidden string" in out.getvalue())
        self.assertFalse("hidden string" in err.getvalue())
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_compat(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        from app_helper.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(SystemExit) as exit:
                        args = list()
                        args.append("cms_helper")
                        args.append("test")
                        args.append("example1")
                        runner.cms("example1", args)
        self.assertTrue("visible string" in out.getvalue())
        self.assertFalse("hidden string" in out.getvalue())
        self.assertFalse("hidden string" in err.getvalue())
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_cms_exception(self):
        try:
            import cms

            raise unittest.SkipTest("django CMS available, skipping test")
        except ImportError:
            pass
        from app_helper.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    with self.assertRaises(ImportError) as exit:
                        args = list()
                        args.append("helper")
                        runner.cms("example1", args)

    def test_runner_cms_argv(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")

        def fake_runner(argv):
            return argv

        from app_helper.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = list()
                    args.append("helper")
                    with patch("app_helper.runner.runner", fake_runner):
                        data = runner.cms("example1", args)
                    self.assertEqual(data, ["helper", "example1", "test", "--cms"])

    def test_runner_argv(self):
        def fake_runner(argv):
            return argv

        from app_helper.test_utils.runners import CapturedOutputRunner

        with patch("django.test.runner.DiscoverRunner", CapturedOutputRunner):
            with work_in(self.basedir):
                with captured_output() as (out, err):
                    args = list()
                    args.append("helper")
                    with patch("app_helper.runner.runner", fake_runner):
                        data = runner.run("example1", args)
                    self.assertEqual(data, ["helper", "example1", "test"])

    def test_setup_cms(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import helper

                settings = runner.setup("example1", helper, use_cms=True, extra_args=["--boilerplate"])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertTrue("aldryn_boilerplates" in settings.INSTALLED_APPS)
        self.assertTrue("cms" in settings.INSTALLED_APPS)

    def test_setup_custom_user(self):
        os.environ["AUTH_USER_MODEL"] = "custom_user.CustomUser"
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import cms_helper_custom

                settings = runner.setup("example1", cms_helper_custom, use_cms=True, extra_args=["--boilerplate"])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertTrue("custom_user" in settings.INSTALLED_APPS)
        self.assertTrue("aldryn_boilerplates" in settings.INSTALLED_APPS)
        self.assertTrue("cms" in settings.INSTALLED_APPS)
        del os.environ["AUTH_USER_MODEL"]

    def test_setup_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                from app_helper.test_utils import helper

                settings = runner.setup("example1", helper, extra_args=[])
        self.assertTrue("example2" in settings.INSTALLED_APPS)
        self.assertFalse("aldryn_boilerplates" in settings.INSTALLED_APPS)
        self.assertFalse("cms" in settings.INSTALLED_APPS)

    def test_testrun_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    try:
                        from django.test.utils import _TestState

                        del _TestState.saved_data
                    except (ImportError, AttributeError):
                        pass
                    args = copy(DEFAULT_ARGS)
                    args["test"] = True
                    args["--cms"] = False
                    args["--runner"] = "runners.CapturedOutputRunner"
                    core(args, self.application)
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_runner_nocms(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    args = list()
                    args.append("helper")
                    args.append("example1")
                    args.append("test")
                    args.append("--extra-settings=helper.py")
                    args.append("--runner=runners.CapturedOutputRunner")
                    runner.run("example1", args)
        self.assertTrue("Ran 14 tests in" in err.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_testrun_native(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                try:
                    from django.test.utils import _TestState

                    del _TestState.saved_data
                except (ImportError, AttributeError):
                    pass
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "test"
                args["--cms"] = False
                args["--native"] = True
                args["--extra-settings"] = "cms_helper_extra_runner.py"
                core(args, self.application)
        self.assertTrue("Ran 14 tests in" in err.getvalue())

    def test_testrun_pytest(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        with work_in(self.basedir):
            with captured_output() as (out, err):
                try:
                    from django.test.utils import _TestState

                    del _TestState.saved_data
                except (ImportError, AttributeError):
                    pass
                args = copy(DEFAULT_ARGS)
                args["<command>"] = "test"
                args["--cms"] = False
                args["--runner"] = "app_helper.pytest_runner.PytestTestRunner"
                args["--extra-settings"] = "helper_no_cms.py"
                core(args, self.application)
        self.assertTrue("collected 15 items" in out.getvalue())
        # warnings will depend on django version and adds too much noise
        self.assertTrue("10 passed, 5 skipped" in out.getvalue())

    def test_runner_pytest(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                with self.assertRaises(SystemExit) as exit:
                    args = list()
                    args.append("helper_no_cms")
                    args.append("example1")
                    args.append("test")
                    args.append("--extra-settings=helper_no_cms.py")
                    args.append("--runner=app_helper.pytest_runner.PytestTestRunner")
                    runner.run("example1", args)
        self.assertTrue("collected 15 items" in out.getvalue())
        # warnings will depend on django version and adds too much noise
        self.assertTrue("10 passed, 5 skipped" in out.getvalue())
        self.assertEqual(exit.exception.code, 0)

    def test_authors(self):
        with work_in(self.basedir):
            with captured_output() as (out, err):
                args = copy(DEFAULT_ARGS)
                args["authors"] = True
                core(args, self.application)
        self.assertTrue("Generating AUTHORS" in out.getvalue())
        self.assertTrue("* Iacopo Spalletti" in out.getvalue())
        self.assertTrue("Authors (" in out.getvalue())

    def test_urls(self):
        try:
            import cms
        except ImportError:
            raise unittest.SkipTest("django CMS not available, skipping test")
        try:
            from django.urls import reverse
        except ImportError:
            from django.core.urlresolvers import reverse
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                core(args, self.application)
                self.assertTrue(reverse("pages-root"))

    def test_urls_nocms(self):
        try:
            from django.urls import reverse, NoReverseMatch
        except ImportError:
            from django.core.urlresolvers import reverse, NoReverseMatch
        with work_in(self.basedir):
            with captured_output() as (out, err):
                shutil.copy(self.poexample, self.pofile)
                args = copy(DEFAULT_ARGS)
                args["makemessages"] = True
                args["--cms"] = False
                core(args, self.application)
                with self.assertRaises(NoReverseMatch):
                    reverse("pages-root")



import pytest


@pytest.fixture(scope="session")
def django_db_setup(
    request,
    django_test_environment,
    django_db_blocker,
    django_db_use_migrations,
    django_db_keepdb,
    django_db_modify_db_settings,
):
    django_db_blocker.unblock()



# -*- coding: utf-8 -*-
from tempfile import mkdtemp

try:
    import djangocms_text_ckeditor  # NOQA

    text_plugin = ["djangocms_text_ckeditor"]
except ImportError:
    text_plugin = []

HELPER_SETTINGS = {
    "TIME_ZONE": "Europe/Rome",
    "INSTALLED_APPS": ["example2", "filer"] + text_plugin,
    "CMS_LANGUAGES": {
        1: [
            {"code": "en", "name": "English", "public": True},
            {"code": "it", "name": "Italiano", "public": True},
            {"code": "fr", "name": "French", "public": True},
        ],
        "default": {"hide_untranslated": False},
    },
    "FILE_UPLOAD_TEMP_DIR": mkdtemp(),
}


def run():
    from app_helper import runner

    runner.cms("example1")


def setup():
    import sys
    from app_helper import runner

    runner.setup("example1", sys.modules[__name__], use_cms=True)


def setup_nocms():
    import sys
    from app_helper import runner

    runner.setup("example1", sys.modules[__name__], use_cms=False)


if __name__ == "__main__":
    run()



# -*- coding: utf-8 -*-
import sys

from django.test.runner import DiscoverRunner

try:
    from django.utils import unittest
except ImportError:
    import unittest


class CapturedOutputRunner(DiscoverRunner):
    def run_suite(self, suite, **kwargs):
        return unittest.TextTestRunner(verbosity=self.verbosity, failfast=self.failfast, stream=sys.stderr).run(suite)


try:
    from django.test.simple import DjangoTestSuiteRunner

    class CapturedOutputSimpleRunner(DjangoTestSuiteRunner):
        def run_suite(self, suite, **kwargs):
            return unittest.TextTestRunner(verbosity=self.verbosity, failfast=self.failfast, stream=sys.stderr).run(
                suite,
            )


except ImportError:
    CapturedOutputSimpleRunner = CapturedOutputRunner



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):
    def forwards(self, orm):
        # Adding model 'ExampleModel1'
        db.create_table(
            "example1_examplemodel1", (("id", self.gf("django.db.models.fields.AutoField")(primary_key=True)),)
        )
        db.send_create_signal("example1", ["ExampleModel1"])

    def backwards(self, orm):
        # Deleting model 'ExampleModel1'
        db.delete_table("example1_examplemodel1")

    models = {
        "example1.examplemodel1": {
            "Meta": {"object_name": "ExampleModel1"},
            "id": ("django.db.models.fields.AutoField", [], {"primary_key": "True"}),
        }
    }

    complete_apps = ["example1"]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from django.contrib.auth.models import AbstractUser


class CustomUser(AbstractUser):
    @property
    def email(self):
        return "some@example.com"



# -*- coding: utf-8 -*-
import pytest
from django.contrib.auth import get_user_model

User = get_user_model()


@pytest.mark.django_db
def test_user():
    user = User.objects.create(username="foo")
    assert user.pk



# -*- coding: utf-8 -*-
import os

try:
    import unittest2 as unittest
except ImportError:
    import unittest

try:
    from django.contrib.auth.models import AnonymousUser
    from app_helper.base_test import BaseTestCase
    from app_helper.utils import get_user_model_labels, CMS_34

    class FakeTests(BaseTestCase):
        _pages_data = (
            {
                "en": {"title": "Page title", "template": "page.html", "publish": True},
                "fr": {"title": "Titre", "publish": True},
                "it": {"title": "Titolo pagina", "publish": False},
            },
            {
                "en": {
                    "title": "Second page",
                    "template": "page.html",
                    "publish": True,
                    "parent": "page-title",
                    "apphook": "Example",
                },
                "fr": {"title": "Deuxieme", "publish": True},
                "it": {"title": "Seconda pagina", "publish": False},
            },
        )

        def test_fake(self):
            self.assertTrue(True)

        def test_pages(self):
            from django.conf import settings

            data = self.get_pages_data()
            self.assertEqual(set(data[0].keys()), set(("en", "fr", "it")))

            if "cms" in settings.INSTALLED_APPS:
                pages = self.get_pages()
                self.assertEqual(len(pages), 2)

        def test_get(self):
            from django.conf import settings

            if "cms" not in settings.INSTALLED_APPS:
                raise unittest.SkipTest("django CMS not available, skipping test")
            from cms.api import add_plugin

            pages = self.get_pages()
            add_plugin(placeholder=pages[0].placeholders.get(slot="content"), plugin_type="FakePlugin", language="en")
            pages[0].publish("en")
            response = self.client.get("/en/")
            self.assertContains(response, "fake text")
            self.assertContains(response, "body{font-weight: bold;}")
            self.assertContains(response, "Page title")

        def test_create_user(self):
            from django.conf import settings

            if "cms" not in settings.INSTALLED_APPS:
                raise unittest.SkipTest("django CMS not available, skipping test")
            user = self.create_user(
                username="random",
                email="random@example.com",
                password="random",
                is_staff=True,
                base_cms_permissions=True,
                permissions=["add_placeholder"],
            )
            self._login_context = self.login_user_context(user)
            self._login_context.user.has_perm("add_placeholdr")
            self._login_context.user.has_perm("add_text")

        def test_login_context(self):
            request = self.get_request(None, "en", path="/en")
            self.assertTrue(request.user, AnonymousUser())

            self._login_context = self.login_user_context(self.user)
            request = self.get_request(None, "en", path="/en")
            self.assertTrue(request.user, self.user)
            self._login_context.__exit__(None, None, None)

            request = self.get_request(None, "en", path="/en")
            self.assertTrue(request.user, AnonymousUser())

            with self.login_user_context(self.user):
                request = self.get_request(None, "en", path="/en")
                self.assertTrue(request.user, self.user)

            request = self.get_request(None, "en", path="/en")
            self.assertTrue(request.user, AnonymousUser())

        def test_requests(self):
            from django.conf import settings

            print("visible string")
            with self.captured_output() as (out, err):
                print("hidden string")
            if "cms" in settings.INSTALLED_APPS:
                pages = self.get_pages()

                request = self.get_request(pages[1], "en")
                self.assertEqual(request.path, "/en/second-page/")
                self.assertEqual(request.META["REQUEST_METHOD"], "GET")
                self.assertEqual(request.current_page, pages[1])

                request = self.get_request(None, "en", path="/cumstom-path/")
                self.assertEqual(request.path, "/cumstom-path/")
                self.assertEqual(request.META["REQUEST_METHOD"], "GET")
                self.assertEqual(request.current_page, None)

                request = self.post_request(pages[1], "en", data={"payload": 1})
                self.assertEqual(request.path, "/en/second-page/")
                self.assertEqual(request.META["REQUEST_METHOD"], "POST")
                self.assertEqual(request.POST.get("payload"), "1")

                request = self.get_page_request(pages[1], lang="en", user=self.user_staff, edit=True,)
                self.assertEqual(request.path, "/en/second-page/")
                self.assertTrue(request.toolbar)
                self.assertEqual(request.META["REQUEST_METHOD"], "GET")

        def test_get_user_model(self):
            if os.environ.get("AUTH_USER_MODEL", "auth.User"):
                user_orm_label, user_model_label = get_user_model_labels()
                self.assertEqual(user_orm_label, "auth.User")
                self.assertEqual(user_model_label, "auth.user")

        def test_create_image(self):
            from filer.models import Image

            image = self.create_filer_image_object()
            self.assertEqual(image.original_filename, self.image_name)
            self.assertEqual(image.width, 800)
            self.assertEqual(image.height, 600)
            self.assertEqual(Image.objects.count(), 1)

        def test_create_filer_image(self):
            from filer.models import Image

            image = BaseTestCase.create_filer_image(self.user, "random.jpg")
            self.assertEqual(image.original_filename, "random.jpg")
            self.assertEqual(image.width, 800)
            self.assertEqual(image.height, 600)
            self.assertEqual(Image.objects.count(), 1)

        def test_create_django_image_object(self):
            image = self.create_django_image_object()
            self.assertEqual(image.name, self.image_name)

        def test_render_plugin(self):
            from django.conf import settings

            if "cms" not in settings.INSTALLED_APPS:
                raise unittest.SkipTest("django CMS not available, skipping test")

            from cms.api import add_plugin

            sample_text = "\nfake text\nen\nPage title\n\n"
            pages = self.get_pages()
            public = pages[0].get_public_object()
            placeholder = pages[0].placeholders.get(slot="content")
            plugin = add_plugin(placeholder=placeholder, plugin_type="FakePlugin", language="en")
            pages[0].publish("en")
            rendered_2 = self.render_plugin(public, "en", plugin)
            if CMS_34:
                context = self.get_plugin_context(pages[0], "en", plugin, edit=False)
                rendered_1 = plugin.render_plugin(context, placeholder)
                self.assertEqual(rendered_2, rendered_1)
            self.assertEqual(rendered_2, sample_text)

        def test_request(self):
            from django.conf import settings

            if "cms" not in settings.INSTALLED_APPS:
                raise unittest.SkipTest("django CMS not available, skipping test")

            pages = self.get_pages()

            request = self.get_request(pages[1], lang="en")
            self.assertIsNone(getattr(request, "toolbar", None))

            request = self.get_page_request(pages[1], user=self.user)
            self.assertIsNotNone(getattr(request, "toolbar", None))

            request = self.get_request(pages[1], "en", use_middlewares=True)
            self.assertIsNotNone(getattr(request, "toolbar", None))
            self.assertIsNotNone(getattr(request, "_messages", None))

            request = self.get_request(pages[1], "en", secure=True)
            self.assertTrue(request.is_secure())

        def test_request_full_middlewares(self):
            # naked request
            request = self.get_request(page=None, lang="en", user=None, path="/en/second-page/", use_middlewares=True)
            self.assertEqual(request.user, AnonymousUser())
            with self.assertRaises(AttributeError):
                self.assertTrue(request.current_page.pk)
            self.assertIsNotNone(getattr(request, "_messages", None))

            # passing user
            request = self.get_request(
                page=None, lang="en", user=self.user, path="/en/second-page/", use_middlewares=True
            )
            self.assertEqual(request.user, self.user)
            with self.assertRaises(AttributeError):
                self.assertTrue(request.current_page.pk)
            self.assertIsNotNone(getattr(request, "_messages", None))

            # logged in user
            with self.login_user_context(self.user):
                request = self.get_request(
                    page=None, lang="en", user=None, path="/en/second-page/", use_middlewares=True
                )
                self.assertEqual(request.user, self.user)
                with self.assertRaises(AttributeError):
                    self.assertTrue(request.current_page.pk)

        def test_request_full_middlewares_cms(self):
            from django.conf import settings

            if "cms" not in settings.INSTALLED_APPS:
                raise unittest.SkipTest("django CMS not available, skipping test")

            pages = self.get_pages()
            # naked request, page and language is derived from path
            request = self.get_request(page=None, lang=None, user=None, path="/en/second-page/", use_middlewares=True)
            self.assertEqual(request.user, AnonymousUser())
            self.assertEqual(request.LANGUAGE_CODE, "en")
            self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
            self.assertIsNotNone(getattr(request, "_messages", None))

            # naked request, page and language is derived from path, user is logged in
            request = self.get_request(
                page=None, lang=None, user=self.user, path="/en/second-page/", use_middlewares=True
            )
            self.assertEqual(request.user, self.user)
            self.assertEqual(request.LANGUAGE_CODE, "en")
            self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
            self.assertIsNotNone(getattr(request, "_messages", None))

            # naked request, path is derived from path
            request = self.get_request(page=pages[1], lang="en", user=self.user, use_middlewares=True)
            self.assertEqual(request.user, self.user)
            self.assertEqual(request.path_info, "/en/second-page/")
            self.assertEqual(request.LANGUAGE_CODE, "en")
            self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())
            self.assertIsNotNone(getattr(request, "_messages", None))

            # logged in user through context manager
            with self.login_user_context(self.user):
                request = self.get_request(
                    page=None, lang=None, user=None, path="/en/second-page/", use_middlewares=True
                )
                self.assertEqual(request.user, self.user)
                self.assertEqual(request.current_page.get_absolute_url(), pages[1].get_absolute_url())


except Exception as e:
    print(e)
    from unittest2 import TestCase

    class FakeTests(TestCase):
        def test_fake(self):
            self.assertTrue(True)



# -*- coding: utf-8 -*-
from tempfile import mkdtemp

HELPER_SETTINGS = {
    "TIME_ZONE": "Europe/Rome",
    "INSTALLED_APPS": ["example2", "filer"],
    "CMS_LANGUAGES": {
        1: [
            {"code": "en", "name": "English", "public": True},
            {"code": "it", "name": "Italiano", "public": True},
            {"code": "fr", "name": "French", "public": True},
        ],
        "default": {"hide_untranslated": False},
    },
    "FILE_UPLOAD_TEMP_DIR": mkdtemp(),
    "TEST_RUNNER": "app_helper.pytest_runner.PytestTestRunner",
    "ALLOWED_HOSTS": ["testserver"],
}


def run():
    from app_helper import runner

    runner.run("example1")


def setup():
    import sys
    from app_helper import runner

    runner.setup("example1", sys.modules[__name__], use_cms=False)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


class Example(CMSApp):
    name = _("Example")
    urls = ["app_helper.urls"]


apphook_pool.register(Example)



# -*- coding: utf-8 -*-
from __future__ import print_function, with_statement

from django.db import models
from django.utils.translation import ugettext_lazy as _


class ExampleModel1(models.Model):
    test_field = models.CharField(max_length=20, default="", verbose_name=_("Test field"))



# -*- coding: utf-8 -*-
try:
    import djangocms_text_ckeditor  # NOQA

    text_plugin = ["djangocms_text_ckeditor"]
except ImportError:
    text_plugin = []

HELPER_SETTINGS = {
    "TIME_ZONE": "Europe/Paris",
    "INSTALLED_APPS": ["some_app"] + text_plugin,
    "TEMPLATE_CONTEXT_PROCESSORS": ["django.core.context_processors.debug"],
    "TEMPLATE_LOADERS": ["admin_tools.template_loaders.Loader"],
    "TOP_MIDDLEWARE_CLASSES": ["top_middleware"],
    "MIDDLEWARE_CLASSES": ["some_middleware"],
    "TOP_INSTALLED_APPS": ["djangocms_admin_style"],
    "ALDRYN_BOILERPLATE_NAME": "legacy",
    "TEMPLATE_DIRS": ["some/dir"],
}



#!/usr/bin/env python
from __future__ import absolute_import, print_function, unicode_literals

import contextlib
import os
import subprocess
import sys
import warnings

from django.utils import autoreload
from django.utils.encoding import force_text
from docopt import DocoptExit, docopt
from six import text_type

from . import __version__
from .utils import (
    DJANGO_1_11,
    _create_db,
    _make_settings,
    create_user,
    ensure_unicoded_and_unique,
    get_user_model,
    persistent_dir,
    temp_dir,
    work_in,
)

__doc__ = """django CMS applications development helper script.

To use a different database, set the DATABASE_URL environment variable to a
dj-database-url compatible value.

Usage:
    django-app-helper <application> test [--failfast] [--migrate] [--no-migrate] [<test-label>...] [--xvfb] [--runner=<test.runner.class>] [--extra-settings=</path/to/settings.py>] [--cms] [--runner-options=<option1>,<option2>] [--native] [--boilerplate] [--persistent] [--persistent-path=<path>] [--verbose=<level>]
    django-app-helper <application> cms_check [--extra-settings=</path/to/settings.py>] [--cms] [--migrate] [--no-migrate] [--boilerplate]
    django-app-helper <application> compilemessages [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate]
    django-app-helper <application> makemessages [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate] [--locale=locale]
    django-app-helper <application> makemigrations [--extra-settings=</path/to/settings.py>] [--cms] [--merge] [--empty] [--dry-run] [--boilerplate] [<extra-applications>...]
    django-app-helper <application> pyflakes [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate]
    django-app-helper <application> authors [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate]
    django-app-helper <application> server [--port=<port>] [--bind=<bind>] [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate] [--migrate] [--no-migrate] [--persistent | --persistent-path=<path>] [--verbose=<level>]
    django-app-helper <application> setup [--extra-settings=</path/to/settings.py>] [--cms] [--boilerplate]
    django-app-helper <application> <command> [options] [--extra-settings=</path/to/settings.py>] [--cms] [--persistent] [--persistent-path=<path>] [--boilerplate] [--migrate] [--no-migrate]

Options:
    -h --help                   Show this screen.
    --version                   Show version.
    --migrate                   Use migrations.
    --no-migrate                Skip south migrations.
    --cms                       Add support for CMS in the project configuration.
    --merge                     Merge migrations
    --failfast                  Stop tests on first failure.
    --native                    Use the native test command, instead of the django-app-helper on
    --boilerplate               Add support for aldryn-boilerplates
    --persistent                Use persistent storage
    --persistent-path=<path>    Persistent storage path
    --locale=locale,-l=locale   Update messgaes for given locale
    --xvfb                      Use a virtual X framebuffer for frontend testing, requires xvfbwrapper to be installed.
    --extra-settings=</path/to/settings.py>     Filesystem path to a custom app_helper file which defines custom settings
    --runner=<test.runner.class>                Dotted path to a custom test runner
    --runner-options=<option1>,<option2>        Comma separated list of command line options for the test runner
    --port=<port>                               Port to listen on [default: 8000].
    --bind=<bind>                               Interface to bind to [default: 127.0.0.1].
    <extra-applications>                        Comma separated list of applications to create migrations for
"""  # NOQA # nopyflakes


def _test_run_worker(test_labels, test_runner, failfast=False, runner_options=None, verbose=1):
    warnings.filterwarnings(
        "error", r"DateTimeField received a naive datetime", RuntimeWarning, r"django\.db\.models\.fields",
    )
    from django.conf import settings
    from django.test.utils import get_runner

    try:
        verbose = int(verbose)
    except (ValueError, TypeError):
        verbose = 1
    runner_options = runner_options or []
    settings.TEST_RUNNER = test_runner
    TestRunner = get_runner(settings)  # NOQA

    if runner_options:
        sys.argv.extend(runner_options.split(","))
    test_runner = TestRunner(verbosity=verbose, interactive=False, failfast=failfast)
    failures = test_runner.run_tests(test_labels)
    return failures


def test(test_labels, application, failfast=False, test_runner=None, runner_options=None, verbose=1):
    """
    Runs the test suite
    :param test_labels: space separated list of test labels
    :param failfast: option to stop the testsuite on the first error
    """
    if not test_labels and "PytestTestRunner" not in test_runner:
        if os.path.exists("tests"):  # pragma: no cover
            test_labels = ["tests"]
        elif os.path.exists(os.path.join(application, "tests")):
            test_labels = ["%s.tests" % application]
    elif type(test_labels) is text_type:
        test_labels = [test_labels]
    runner_options = runner_options or []
    return _test_run_worker(test_labels, test_runner, failfast, runner_options, verbose)


def compilemessages(application):
    """
    Compiles locale messages
    """
    from django.core.management import call_command

    with work_in(application):
        if DJANGO_1_11:
            call_command("compilemessages", all=True)
        else:
            call_command("compilemessages")


def makemessages(application, locale):
    """
    Updates the locale message files
    """
    from django.core.management import call_command

    if not locale:
        locale = "en"
    with work_in(application):
        call_command("makemessages", locale=(locale,))


def cms_check(migrate_cmd=False):
    """
    Runs the django CMS ``cms check`` command
    """
    from django.core.management import call_command

    try:
        import cms  # NOQA # nopyflakes

        _create_db(migrate_cmd)
        call_command("cms", "check")
    except ImportError:
        print("cms_check available only if django CMS is installed")


def makemigrations(application, merge=False, dry_run=False, empty=False, extra_applications=None):
    """
    Generate migrations
    """
    from django.core.management import call_command

    apps = [application]
    if extra_applications:
        if isinstance(extra_applications, text_type):
            apps += [extra_applications]
        elif isinstance(extra_applications, list):
            apps += extra_applications

    for app in apps:
        call_command("makemigrations", *(app,), merge=merge, dry_run=dry_run, empty=empty)


def generate_authors():
    """
    Updates the authors list
    """
    print("Generating AUTHORS")

    # Get our list of authors
    print("Collecting author names")
    r = subprocess.Popen(["git", "log", "--use-mailmap", "--format=%aN"], stdout=subprocess.PIPE)
    seen_authors = []
    authors = []
    for authfile in ("AUTHORS", "AUTHORS.rst"):
        if os.path.exists(authfile):
            break
    with open(authfile, "r") as f:
        for line in f.readlines():
            if line.startswith("*"):
                author = force_text(line).strip("* \n")
                if author.lower() not in seen_authors:
                    seen_authors.append(author.lower())
                    authors.append(author)
    for author in r.stdout.readlines():
        author = force_text(author).strip()
        if author.lower() not in seen_authors:
            seen_authors.append(author.lower())
            authors.append(author)

    # Sort our list of Authors by their case insensitive name
    authors = sorted(authors, key=lambda x: x.lower())

    # Write our authors to the AUTHORS file
    print("Authors (%s):\n\n\n* %s" % (len(authors), "\n* ".join(authors)))


def static_analisys(application):
    """
    Performs a pyflakes static analysis with the same configuration as
    django CMS testsuite
    """
    try:
        from cms.test_utils.util.static_analysis import pyflakes

        application_module = __import__(application)
        report = pyflakes((application_module,))
        if type(report) == tuple:
            assert report[0] == 0
        else:
            assert report == 0
    except ImportError:
        print(
            "Static analysis available only if django CMS and pyflakes are installed.\n"
            "Install django-app-helper[pyflakes] to fix this."
        )


def server(bind="127.0.0.1", port=8000, migrate_cmd=False, verbose=1):  # pragma: no cover
    try:
        from channels.log import setup_logger
        from channels.management.commands import runserver

        logger = setup_logger("django.channels", 1)
        use_channels = True
    except ImportError:
        from django.contrib.staticfiles.management.commands import runserver

        use_channels = False
        logger = None

    if os.environ.get("RUN_MAIN") != "true":
        _create_db(migrate_cmd)
        User = get_user_model()  # NOQA
        if not User.objects.filter(is_superuser=True).exists():
            usr = create_user("admin", "admin@admin.com", "admin", is_staff=True, is_superuser=True)
            print("")
            print("A admin user (username: %s, password: admin) " "has been created." % usr.get_username())
            print("")
    rs = runserver.Command()
    try:
        from django.core.management.base import OutputWrapper

        rs.stdout = OutputWrapper(sys.stdout)
        rs.stderr = OutputWrapper(sys.stderr)
    except ImportError:
        rs.stdout = sys.stdout
        rs.stderr = sys.stderr
    rs.use_ipv6 = False
    rs._raw_ipv6 = False
    rs.addr = bind
    rs.port = port
    if logger:
        rs.logger = logger
    if use_channels:
        rs.http_timeout = 60
        rs.websocket_handshake_timeout = 5
    try:
        autoreload.run_with_reloader(
            rs.inner_run,
            **{
                "addrport": "%s:%s" % (bind, port),
                "insecure_serving": True,
                "use_static_handler": True,
                "use_threading": True,
                "verbosity": verbose,
                "use_reloader": True,
            }
        )
    except AttributeError:
        autoreload.main(
            rs.inner_run,
            kwargs={
                "addrport": "%s:%s" % (bind, port),
                "insecure_serving": True,
                "use_static_handler": True,
                "use_threading": True,
                "verbosity": verbose,
                "use_reloader": True,
            }
        )


def setup_env(settings):
    return settings


def _map_argv(argv, application_module):
    try:
        # by default docopt uses sys.argv[1:]; ensure correct args passed
        args = docopt(__doc__, argv=argv[1:], version=application_module.__version__)
        if argv[2] == "help":
            raise DocoptExit()
    except DocoptExit:
        if argv[2] == "help":
            raise
        args = docopt(__doc__, argv[1:3], version=application_module.__version__)
    args["--cms"] = "--cms" in argv
    args["--persistent"] = "--persistent" in argv
    for arg in argv:
        if arg.startswith("--extra-settings="):
            args["--extra-settings"] = arg.split("=")[1]
        if arg.startswith("--runner="):
            args["--runner"] = arg.split("=")[1]
        if arg.startswith("--persistent-path="):
            args["--persistent-path"] = arg.split("=")[1]
            args["--persistent"] = True
    args["options"] = [argv[0]] + argv[2:]
    if args["test"] and "--native" in args["options"]:
        args["test"] = False
        args["<command>"] = "test"
        args["options"].remove("--native")
    return args


def core(args, application):
    from django.conf import settings

    # configure django
    warnings.filterwarnings(
        "error", r"DateTimeField received a naive datetime", RuntimeWarning, r"django\.db\.models\.fields",
    )
    if args["--persistent"]:
        create_dir = persistent_dir
        if args["--persistent-path"]:
            parent_path = args["--persistent-path"]
        else:
            parent_path = "data"
    else:
        create_dir = temp_dir
        parent_path = "/dev/shm"

    with create_dir("static", parent_path) as STATIC_ROOT:  # NOQA
        with create_dir("media", parent_path) as MEDIA_ROOT:  # NOQA
            args["MEDIA_ROOT"] = MEDIA_ROOT
            args["STATIC_ROOT"] = STATIC_ROOT
            if args["cms_check"]:
                args["--cms"] = True

            if args["<command>"]:
                from django.core.management import execute_from_command_line

                options = [
                    option
                    for option in args["options"]
                    if (
                        option != "--cms"
                        and "--extra-settings" not in option
                        and not option.startswith("--persistent")
                    )
                ]
                _make_settings(args, application, settings, STATIC_ROOT, MEDIA_ROOT)
                execute_from_command_line(options)

            else:
                _make_settings(args, application, settings, STATIC_ROOT, MEDIA_ROOT)
                # run
                if args["test"]:
                    if args["--runner"]:
                        runner = args["--runner"]
                    else:
                        runner = settings.TEST_RUNNER

                    # make "Address already in use" errors less likely, see Django
                    # docs for more details on this env variable.
                    os.environ.setdefault("DJANGO_LIVE_TEST_SERVER_ADDRESS", "localhost:8000-9000")
                    if args["--xvfb"]:  # pragma: no cover
                        import xvfbwrapper

                        context = xvfbwrapper.Xvfb(width=1280, height=720)
                    else:

                        @contextlib.contextmanager
                        def null_context():
                            yield

                        context = null_context()

                    with context:
                        num_failures = test(
                            args["<test-label>"],
                            application,
                            args["--failfast"],
                            runner,
                            args["--runner-options"],
                            args.get("--verbose", 1),
                        )
                        sys.exit(num_failures)
                elif args["server"]:
                    server(args["--bind"], args["--port"], args.get("--migrate", True), args.get("--verbose", 1))
                elif args["cms_check"]:
                    cms_check(args.get("--migrate", True))
                elif args["compilemessages"]:
                    compilemessages(application)
                elif args["makemessages"]:
                    makemessages(application, locale=args["--locale"])
                elif args["makemigrations"]:
                    makemigrations(
                        application,
                        merge=args["--merge"],
                        dry_run=args["--dry-run"],
                        empty=args["--empty"],
                        extra_applications=args["<extra-applications>"],
                    )
                elif args["pyflakes"]:
                    return static_analisys(application)
                elif args["authors"]:
                    return generate_authors()
                elif args["setup"]:
                    return setup_env(settings)


def main(argv=sys.argv):  # pragma: no cover
    # Command is executed in the main directory of the plugin, and we must
    # include it in the current path for the imports to work
    sys.path.insert(0, ".")
    if len(argv) > 1:
        application = argv[1]
        # ensure that argv, are unique and the same type as doc string
        argv = ensure_unicoded_and_unique(argv, application)
        application_module = __import__(application)
        args = _map_argv(argv, application_module)
        return core(args=args, application=application)
    else:
        args = docopt(__doc__, version=__version__)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals


def get_default_settings(CMS_APP, CMS_PROCESSORS, CMS_MIDDLEWARE, CMS_APP_STYLE, URLCONF, application):  # NOQA
    return dict(  # NOQA
        INSTALLED_APPS=[
            "django.contrib.contenttypes",
            "django.contrib.auth",
            "django.contrib.sessions",
            "django.contrib.sites",
            "django.contrib.staticfiles",
        ]
        + CMS_APP_STYLE
        + ["django.contrib.admin", "app_helper.test_data", "django.contrib.messages"]
        + CMS_APP,
        DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}},
        TEMPLATE_LOADERS=[
            "django.template.loaders.filesystem.Loader",
            "django.template.loaders.app_directories.Loader",
        ],
        STATICFILES_FINDERS=[
            "django.contrib.staticfiles.finders.FileSystemFinder",
            "django.contrib.staticfiles.finders.AppDirectoriesFinder",
        ],
        TEMPLATE_CONTEXT_PROCESSORS=[
            "django.contrib.auth.context_processors.auth",
            "django.contrib.messages.context_processors.messages",
            "django.core.context_processors.i18n",
            "django.core.context_processors.csrf",
            "django.core.context_processors.debug",
            "django.core.context_processors.tz",
            "django.core.context_processors.request",
            "django.core.context_processors.media",
            "django.core.context_processors.static",
        ]
        + CMS_PROCESSORS,
        MIDDLEWARE_CLASSES=[
            "django.middleware.http.ConditionalGetMiddleware",
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
            "django.contrib.messages.middleware.MessageMiddleware",
            "django.middleware.csrf.CsrfViewMiddleware",
            "django.middleware.locale.LocaleMiddleware",
            "django.middleware.common.CommonMiddleware",
        ]
        + CMS_MIDDLEWARE,
        ROOT_URLCONF=URLCONF,
        SITE_ID=1,
        LANGUAGE_CODE="en",
        LANGUAGES=(("en", "English"),),
        STATIC_URL="/static/",
        MEDIA_URL="/media/",
        DEBUG=True,
        CMS_TEMPLATES=(("fullwidth.html", "Fullwidth"), ("page.html", "Normal page")),
        PASSWORD_HASHERS=("django.contrib.auth.hashers.MD5PasswordHasher",),
        MIGRATION_MODULES={},
        EMAIL_BACKEND="django.core.mail.backends.locmem.EmailBackend",
    )


def get_boilerplates_settings():
    return {
        "STATICFILES_FINDERS": ["aldryn_boilerplates.staticfile_finders.AppDirectoriesFinder"],
        "TEMPLATE_LOADERS": ["aldryn_boilerplates.template_loaders.AppDirectoriesLoader"],
        "TEMPLATE_CONTEXT_PROCESSORS": ["aldryn_boilerplates.context_processors.boilerplate"],
        "ALDRYN_BOILERPLATE_NAME": "bootstrap3",
        "INSTALLED_APPS": ["aldryn_boilerplates"],
    }



# -*- coding: utf-8 -*-
from app_helper.test_utils.helper import HELPER_SETTINGS

HELPER_SETTINGS["TEMPLATE_CONTEXT_PROCESSORS"] = ["django.core.context_processors.debug"]
HELPER_SETTINGS["TEST_RUNNER"] = "runners.CapturedOutputRunner"
HELPER_SETTINGS["INSTALLED_APPS"].remove("djangocms_text_ckeditor")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import inspect
import os.path
import sys


def run(app, argv=sys.argv, extra_args=None):
    """
    Run commands in a plain django environment

    :param app: application
    :param argv: arguments (default to sys.argv)
    :param extra_args: list of extra arguments
    """
    if app not in argv[:2]:
        # app is automatically added if not present
        argv.insert(1, app)
    if len(argv) < 3 and "test" not in argv[:2]:
        # test argument is given if not argument is passed
        argv.insert(2, "test")
    if extra_args:
        argv.extend(extra_args)
    return runner(argv)


def cms(app, argv=sys.argv, extra_args=None):
    """
    Run commands in a django cMS environment

    :param app: application
    :param argv: arguments (default to sys.argv)
    :param extra_args: list of extra arguments
    """
    try:
        import cms  # NOQA  # nopyflakes
    except ImportError:
        print("runner.cms is available only if django CMS is installed")
        raise
    if app not in argv[:2]:
        # app is automatically added if not present
        argv.insert(1, app)
    if len(argv) < 3 and "test" not in argv[:2]:
        # test argument is given if not argument is passed
        argv.insert(2, "test")
    if "--cms" not in argv:
        # this is the cms runner, just add the cms argument
        argv.append("--cms")
    if extra_args:
        argv.extend(extra_args)
    return runner(argv)


def setup(app, helper_module, extra_args=None, use_cms=False):
    """
    Setup the Django / django CMS environment and return the environment settings.

    :param app: application
    :param helper_module: helper module
    :param extra_args: list of extra arguments
    :param use_cms: setup a django CMS environemtn
    :return: Django settings module
    """
    helper = helper_module.__file__
    argv = [os.path.basename(helper), app, "setup", "--extra-settings={0}".format(helper)]
    if use_cms:
        argv.append("--cms")
    if extra_args:
        argv.extend(extra_args)
    return runner(argv)


def runner(argv):
    from . import HELPER_FILE
    from .main import main

    # This is a hackish way to get the caller file which is the file
    # which contains the HELPER_SETTINGS
    helper = os.path.abspath(inspect.getframeinfo(inspect.stack()[2][0]).filename)
    # check if extra settings has been passed
    # if not, user the helper file
    extra_settings = any(map(lambda x: x.startswith("--extra-settings="), argv))
    if os.path.basename(helper) not in (HELPER_FILE,) and not extra_settings:
        argv.append("--extra-settings=%s" % helper)
    return main(argv)



# -*- coding: utf-8 -*-
"""
This is a compatibility package to allow legacy import to work without any changes
"""
from __future__ import absolute_import, print_function, unicode_literals

import sys

# clone app_helper in djangocms_helper module
import app_helper  # NOQA

sys.modules[__name__] = sys.modules["app_helper"]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import contextlib
import os
import shutil
import sys
from distutils.version import LooseVersion
from tempfile import mkdtemp

import django
import six
from django.core.management import call_command
from django.urls import clear_url_caches
from django.utils.functional import empty

from . import HELPER_FILE

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


try:
    import cms  # NOQA

    CMS = True
    CMS_40 = LooseVersion("4.0") <= LooseVersion(cms.__version__) < LooseVersion("4.1")
    CMS_38 = LooseVersion("3.8") <= LooseVersion(cms.__version__) < LooseVersion("3.9")
    CMS_37 = LooseVersion("3.7") <= LooseVersion(cms.__version__) < LooseVersion("3.8")
    CMS_36 = LooseVersion("3.6") <= LooseVersion(cms.__version__) < LooseVersion("3.7")
    CMS_35 = LooseVersion("3.5") <= LooseVersion(cms.__version__) < LooseVersion("3.6")
    CMS_34 = LooseVersion("3.4") <= LooseVersion(cms.__version__) < LooseVersion("3.5")
    CMS_33 = LooseVersion("3.3") <= LooseVersion(cms.__version__) < LooseVersion("3.4")
    CMS_32 = LooseVersion("3.2") <= LooseVersion(cms.__version__) < LooseVersion("3.3")
    CMS_31 = LooseVersion("3.1") <= LooseVersion(cms.__version__) < LooseVersion("3.2")
    CMS_30 = LooseVersion("3.0") <= LooseVersion(cms.__version__) < LooseVersion("3.1")
except ImportError:  # pragma: no cover
    CMS = False
    CMS_40 = False
    CMS_38 = False
    CMS_37 = False
    CMS_36 = False
    CMS_35 = False
    CMS_34 = False
    CMS_33 = False
    CMS_32 = False
    CMS_31 = False
    CMS_30 = False

DJANGO_1_4 = LooseVersion(django.get_version()) < LooseVersion("1.5")
DJANGO_1_5 = LooseVersion(django.get_version()) < LooseVersion("1.6")
DJANGO_1_6 = LooseVersion(django.get_version()) < LooseVersion("1.7")
DJANGO_1_7 = LooseVersion(django.get_version()) < LooseVersion("1.8")
DJANGO_1_8 = LooseVersion(django.get_version()) < LooseVersion("1.9")
DJANGO_1_9 = LooseVersion(django.get_version()) < LooseVersion("1.10")
DJANGO_1_10 = LooseVersion(django.get_version()) < LooseVersion("1.11")
DJANGO_1_11 = LooseVersion(django.get_version()) < LooseVersion("2.0")
DJANGO_2_0 = LooseVersion(django.get_version()) < LooseVersion("2.1")
DJANGO_2_1 = LooseVersion(django.get_version()) < LooseVersion("2.2")
DJANGO_2_2 = LooseVersion(django.get_version()) < LooseVersion("3.0")
DJANGO_3_0 = LooseVersion(django.get_version()) < LooseVersion("3.1")
DJANGO_3_1 = LooseVersion(django.get_version()) < LooseVersion("3.2")
DJANGO_3_2 = LooseVersion(django.get_version()) < LooseVersion("4.0")


def load_from_file(module_path):
    """
    Load a python module from its absolute filesystem path

    Borrowed from django-cms
    """
    from imp import load_module, PY_SOURCE

    imported = None
    if module_path:
        with open(module_path, "r") as openfile:
            imported = load_module("mod", openfile, module_path, ("imported", "r", PY_SOURCE))
    return imported


@contextlib.contextmanager
def work_in(dirname=None):
    """
    Context manager version of os.chdir. When exited, returns to the working
    directory prior to entering.

    Grabbed from cookiecutter, thanks audreyr!
    """
    curdir = os.getcwd()
    try:
        if dirname is not None:
            if dirname not in sys.path:
                sys.path.insert(0, dirname)
            os.chdir(dirname)
        yield
    finally:
        os.chdir(curdir)


@contextlib.contextmanager
def captured_output():
    with patch("sys.stdout", new_callable=six.StringIO) as out:
        with patch("sys.stderr", new_callable=six.StringIO) as err:
            yield out, err


# Borrowed from django CMS codebase
@contextlib.contextmanager
def temp_dir(suffix="", container="/dev/shm/"):
    name = make_temp_dir(suffix, container)
    yield name
    shutil.rmtree(name)


def make_temp_dir(suffix="", container="/dev/shm/"):
    if os.path.exists(container):
        return mkdtemp(suffix=suffix, dir=container)
    return mkdtemp(suffix=suffix)


@contextlib.contextmanager
def persistent_dir(suffix, container="data"):
    name = os.path.abspath(os.path.join(container, suffix))
    if not os.path.exists(name):
        os.makedirs(name)
    yield name


class DisableMigrations(object):
    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return None


def _reset_django(settings):
    """
    Hackish way to reset the django instance settings and AppConfig
    :param settings: django settings module
    """
    if settings._wrapped != empty:
        clear_url_caches()
        from django.apps import apps

        apps.clear_cache()
        settings._wrapped = empty
        clear_url_caches()


def extend_settings(settings, extra_settings, key, insertion_point):
    for item in extra_settings[key]:
        if item not in settings[key]:
            settings[key].insert(settings[key].index(insertion_point), item)
    return settings


def _make_settings(args, application, settings, STATIC_ROOT, MEDIA_ROOT):  # NOQA
    """
    Setup the Django settings
    :param args: docopt arguments
    :param application: application module name
    :param settings: Django settings module
    :param STATIC_ROOT: static root directory
    :param MEDIA_ROOT: media root directory
    :return:
    """
    import dj_database_url
    from .default_settings import get_default_settings, get_boilerplates_settings

    try:
        extra_settings_file = args.get("--extra-settings")
        if not extra_settings_file:
            extra_settings_file = HELPER_FILE
        if extra_settings_file[-3:] != ".py":
            filename, __ = os.path.splitext(extra_settings_file)
            extra_settings_file = "{0}.py".format(filename)
        extra_settings = load_from_file(extra_settings_file).HELPER_SETTINGS
    except (IOError, AttributeError):
        extra_settings = None
    default_name = ":memory:" if args["test"] else "local.sqlite"
    db_url = os.environ.get("DATABASE_URL", "sqlite://localhost/%s" % default_name)
    configs = {
        "DATABASES": {"default": dj_database_url.parse(db_url)},
        "STATIC_ROOT": STATIC_ROOT,
        "MEDIA_ROOT": MEDIA_ROOT,
        "USE_TZ": True,
        "USE_CMS": args["--cms"],
        "BASE_APPLICATION": application,
    }

    if configs["USE_CMS"] or getattr(extra_settings, "USE_CMS", False):
        CMS_APPS = [  # NOQA
            "cms",
            "menus",
            "sekizai",
        ]
        CMS_APP_STYLE = ["djangocms_admin_style"]  # NOQA
        CMS_PROCESSORS = [  # NOQA
            "cms.context_processors.cms_settings",
            "sekizai.context_processors.sekizai",
        ]
        CMS_MIDDLEWARE = [  # NOQA
            "cms.middleware.language.LanguageCookieMiddleware",
            "cms.middleware.user.CurrentUserMiddleware",
            "cms.middleware.page.CurrentPageMiddleware",
            "cms.middleware.toolbar.ToolbarMiddleware",
        ]
        if not CMS_31 and args["server"]:
            CMS_MIDDLEWARE.append("cms.middleware.utils.ApphookReloadMiddleware")
        URLCONF = "app_helper.urls"  # NOQA
    else:
        CMS_APPS = []  # NOQA
        CMS_APP_STYLE = []  # NOQA
        CMS_MIDDLEWARE = []  # NOQA
        CMS_PROCESSORS = []  # NOQA
        URLCONF = "app_helper.urls"  # NOQA

    default_settings = get_default_settings(
        CMS_APPS, CMS_PROCESSORS, CMS_MIDDLEWARE, CMS_APP_STYLE, URLCONF, application,
    )
    migrate = args.get("--migrate") or not args.get("--no-migrate")
    default_settings.update(configs)

    if extra_settings:
        apps = extra_settings.pop("INSTALLED_APPS", [])
        apps_top = extra_settings.pop("TOP_INSTALLED_APPS", [])
        template_processors = extra_settings.pop("TEMPLATE_CONTEXT_PROCESSORS", [])
        template_loaders = extra_settings.pop("TEMPLATE_LOADERS", [])
        template_dirs = extra_settings.pop("TEMPLATE_DIRS", [])
        middleware = extra_settings.pop("MIDDLEWARE_CLASSES", [])
        middleware_top = extra_settings.pop("TOP_MIDDLEWARE_CLASSES", [])
        default_settings.update(extra_settings)
        for app in apps_top:
            default_settings["INSTALLED_APPS"].insert(0, app)
        default_settings["INSTALLED_APPS"].extend(apps)
        default_settings["TEMPLATE_CONTEXT_PROCESSORS"].extend(template_processors)
        default_settings["TEMPLATE_LOADERS"].extend(template_loaders)
        if "TEMPLATE_DIRS" not in default_settings:
            default_settings["TEMPLATE_DIRS"] = []
        default_settings["TEMPLATE_DIRS"].extend(template_dirs)
        default_settings["MIDDLEWARE_CLASSES"].extend(middleware)
        for middleware in middleware_top:
            default_settings["MIDDLEWARE_CLASSES"].insert(0, middleware)

    if "cms" in default_settings["INSTALLED_APPS"]:
        if "treebeard" not in default_settings["INSTALLED_APPS"]:
            default_settings["INSTALLED_APPS"].append("treebeard")
    if "filer" in default_settings["INSTALLED_APPS"] and "mptt" not in default_settings["INSTALLED_APPS"]:
        default_settings["INSTALLED_APPS"].append("mptt")
    if "filer" in default_settings["INSTALLED_APPS"] and "easy_thumbnails" not in default_settings["INSTALLED_APPS"]:
        default_settings["INSTALLED_APPS"].append("easy_thumbnails")

    if args["--boilerplate"]:
        boilerplate_settings = get_boilerplates_settings()

        # Do not override helper settings with defaults.
        if "ALDRYN_BOILERPLATE_NAME" in default_settings.keys():
            del boilerplate_settings["ALDRYN_BOILERPLATE_NAME"]

        default_settings = extend_settings(
            default_settings,
            boilerplate_settings,
            "STATICFILES_FINDERS",
            "django.contrib.staticfiles.finders.AppDirectoriesFinder",
        )
        del boilerplate_settings["STATICFILES_FINDERS"]

        default_settings = extend_settings(
            default_settings,
            boilerplate_settings,
            "TEMPLATE_LOADERS",
            "django.template.loaders.app_directories.Loader",
        )
        del boilerplate_settings["TEMPLATE_LOADERS"]

        for setting in ("INSTALLED_APPS", "TEMPLATE_CONTEXT_PROCESSORS"):
            default_settings[setting].extend(boilerplate_settings[setting])
            del boilerplate_settings[setting]
        default_settings.update(boilerplate_settings)

    default_settings["TEMPLATES"] = [
        {
            "NAME": "django",
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "OPTIONS": {
                "context_processors": [
                    template_processor.replace("django.core", "django.template")
                    for template_processor in default_settings.pop("TEMPLATE_CONTEXT_PROCESSORS")
                ],
                "loaders": default_settings.pop("TEMPLATE_LOADERS"),
            },
        },
    ]
    if "TEMPLATE_DIRS" in default_settings:
        default_settings["TEMPLATES"][0]["DIRS"] = default_settings.pop("TEMPLATE_DIRS")

    # Support for custom user models
    if "AUTH_USER_MODEL" in os.environ:
        custom_user_app = os.environ["AUTH_USER_MODEL"].rpartition(".")[0]
        custom_user_model = ".".join(os.environ["AUTH_USER_MODEL"].split(".")[-2:])
        if "cms" in default_settings["INSTALLED_APPS"]:
            default_settings["INSTALLED_APPS"].insert(default_settings["INSTALLED_APPS"].index("cms"), custom_user_app)
        else:
            default_settings["INSTALLED_APPS"].insert(
                default_settings["INSTALLED_APPS"].index("django.contrib.auth") + 1, custom_user_app,
            )
        default_settings["AUTH_USER_MODEL"] = custom_user_model

    if args["test"]:
        default_settings["SESSION_ENGINE"] = "django.contrib.sessions.backends.cache"
    if application not in default_settings["INSTALLED_APPS"]:
        default_settings["INSTALLED_APPS"].append(application)

    if not migrate:
        default_settings["MIGRATION_MODULES"] = DisableMigrations()

    if "MIDDLEWARE" not in default_settings:
        default_settings["MIDDLEWARE"] = default_settings["MIDDLEWARE_CLASSES"]
        del default_settings["MIDDLEWARE_CLASSES"]

    _reset_django(settings)
    settings.configure(**default_settings)
    django.setup()
    reload_urls(settings, cms_apps=False)
    return settings


def reload_urls(settings, urlconf=None, cms_apps=True):
    if "cms.urls" in sys.modules:
        six.moves.reload_module(sys.modules["cms.urls"])
    if urlconf is None:
        urlconf = settings.ROOT_URLCONF
    if urlconf in sys.modules:
        six.moves.reload_module(sys.modules[urlconf])
    clear_url_caches()
    if cms_apps:
        from cms.appresolver import clear_app_resolvers, get_app_patterns

        clear_app_resolvers()
        get_app_patterns()


def _create_db(migrate_cmd=False):
    call_command("migrate")


def get_user_model():
    from django.contrib.auth import get_user_model

    return get_user_model()


def create_user(
    username, email, password, is_staff=False, is_superuser=False, base_cms_permissions=False, permissions=None,
):
    from django.contrib.auth.models import Permission

    User = get_user_model()  # NOQA

    try:
        if User.USERNAME_FIELD == "email":
            user = User.objects.get(**{User.USERNAME_FIELD: email})
        else:
            user = User.objects.get(**{User.USERNAME_FIELD: username})
    except User.DoesNotExist:
        user = User()

    if User.USERNAME_FIELD != "email":
        setattr(user, User.USERNAME_FIELD, username)

    try:
        user.email = email
    except AttributeError:
        pass
    user.set_password(password)
    if is_superuser:
        user.is_superuser = True
    if is_superuser or is_staff:
        user.is_staff = True
    user.is_active = True
    user.save()
    if user.is_staff and not is_superuser and base_cms_permissions:
        user.user_permissions.add(Permission.objects.get(codename="add_text"))
        user.user_permissions.add(Permission.objects.get(codename="delete_text"))
        user.user_permissions.add(Permission.objects.get(codename="change_text"))
        user.user_permissions.add(Permission.objects.get(codename="publish_page"))

        user.user_permissions.add(Permission.objects.get(codename="add_page"))
        user.user_permissions.add(Permission.objects.get(codename="change_page"))
        user.user_permissions.add(Permission.objects.get(codename="delete_page"))
    if is_staff and not is_superuser and permissions:
        for permission in permissions:
            user.user_permissions.add(Permission.objects.get(codename=permission))
    return user


def get_user_model_labels():
    User = get_user_model()  # NOQA

    user_orm_label = "%s.%s" % (User._meta.app_label, User._meta.object_name)
    user_model_label = "%s.%s" % (User._meta.app_label, User._meta.model_name)
    return user_orm_label, user_model_label


class UserLoginContext(object):
    def __init__(self, testcase, user, password=None):
        self.testcase = testcase
        self.user = user
        if password is None:
            password = getattr(user, get_user_model().USERNAME_FIELD)
        self.password = password

    def __enter__(self):
        loginok = self.testcase.client.login(
            username=getattr(self.user, get_user_model().USERNAME_FIELD), password=self.password,
        )
        self.testcase.assertTrue(loginok)
        self.testcase._login_context = self

    def __exit__(self, exc, value, tb):
        self.testcase._login_context = None
        self.testcase.client.logout()


def ensure_unicoded_and_unique(args_list, application):
    """
    Iterate over args_list, make it unicode if needed and ensure that there
    are no duplicates.
    Returns list of unicoded arguments in the same order.
    """
    unicoded_args = []
    for argument in args_list:
        argument = six.u(argument) if not isinstance(argument, six.text_type) else argument
        if argument not in unicoded_args or argument == application:
            unicoded_args.append(argument)
    return unicoded_args



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

__version__ = "2.2.0"
__author__ = "Iacopo Spalletti <i.spalletti@nephila.it>"
__all__ = ["runner"]

HELPER_FILE = "helper.py"



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import os.path
from collections import OrderedDict
from contextlib import contextmanager
from copy import deepcopy
from tempfile import mkdtemp

from django.conf import settings
from django.core.handlers.base import BaseHandler
from django.http import SimpleCookie
from django.test import RequestFactory, TestCase, TransactionTestCase
from django.utils.functional import SimpleLazyObject
from six import StringIO

from .utils import UserLoginContext, create_user, get_user_model, reload_urls, temp_dir

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch


class BaseTestCaseMixin(object):
    """
    Utils mixin that provides some helper methods to setup and interact with
    Django testing framework.
    """

    request_factory = None
    user = None
    user_staff = None
    user_normal = None
    site_1 = None
    languages = None
    _login_context = None
    image_name = "test_image.jpg"

    #: Username for auto-generated superuser
    _admin_user_username = "admin"
    #: Password for auto-generated superuser
    _admin_user_password = "admin"
    #: Email for auto-generated superuser
    _admin_user_email = "admin@admin.com"

    #: Username for auto-generated staff user
    _staff_user_username = "staff"
    #: Password for auto-generated staff user
    _staff_user_password = "staff"
    #: Email for auto-generated staff user
    _staff_user_email = "staff@admin.com"

    #: Username for auto-generated non-staff user
    _user_user_username = "normal"
    #: Password for auto-generated non-staff user
    _user_user_password = "normal"
    #: Email for auto-generated non-staff user
    _user_user_email = "user@admin.com"

    _pages_data = ()
    """
    List of pages data for the different languages.

    Each item of the list is a dictionary containing the attributes
    (as accepted by ``cms.api.create_page``) of the page to be created.

    The first language will be created with ``cms.api.create_page`` the following
    languages using ``cms.api.create_title``

    Example:
        Single page created in en, fr, it languages::

            _pages_data = (
                {
                    'en': {'title': 'Page title', 'template': 'page.html', 'publish': True},
                    'fr': {'title': 'Titre', 'publish': True},
                    'it': {'title': 'Titolo pagina', 'publish': False}
                },
            )

    """

    @classmethod
    def setUpClass(cls):
        from django.contrib.sites.models import Site

        cls.request_factory = RequestFactory()
        cls.user = create_user(
            cls._admin_user_username,
            cls._admin_user_email,
            cls._admin_user_password,
            is_staff=True,
            is_superuser=True,
        )
        cls.user_staff = create_user(
            cls._staff_user_username,
            cls._staff_user_email,
            cls._staff_user_password,
            is_staff=True,
            is_superuser=False,
        )
        cls.user_normal = create_user(
            cls._user_user_username, cls._user_user_email, cls._user_user_password, is_staff=False, is_superuser=False,
        )
        cls.site_1 = Site.objects.all().first()

        try:
            from cms.utils import get_language_list

            cls.languages = get_language_list()
        except ImportError:
            cls.languages = [x[0] for x in settings.LANGUAGES]
        super(BaseTestCaseMixin, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        super(BaseTestCaseMixin, cls).tearDownClass()
        User = get_user_model()  # NOQA
        User.objects.all().delete()

    @contextmanager
    def temp_dir(self):
        """
        Context manager to operate on a temporary directory
        """
        yield temp_dir()

    def reload_model(self, obj):
        """
        Reload a models instance from database
        :param obj: model instance to reload
        :return: the reloaded model instance
        """
        return obj.__class__.objects.get(pk=obj.pk)

    @staticmethod
    def reload_urlconf(urlconf=None):
        reload_urls(settings, urlconf)

    def login_user_context(self, user, password=None):
        """
        Context manager to make logged in requests
        :param user: user username
        :param password: user password (if omitted, username is used)
        """
        return UserLoginContext(self, user, password)

    def create_user(
        self,
        username,
        email,
        password,
        is_staff=False,
        is_superuser=False,
        base_cms_permissions=False,
        permissions=None,
    ):
        """
        Creates a user with the given properties

        :param username: Username
        :param email: Email
        :param password: password
        :param is_staff: Staff status
        :param is_superuser: Superuser status
        :param base_cms_permissions: Base django CMS permissions
        :param permissions: Other permissions
        :return: User instance
        """
        return create_user(username, email, password, is_staff, is_superuser, base_cms_permissions, permissions)

    def get_pages_data(self):
        """
        Construct a list of pages in the different languages available for the
        project. Default implementation is to return the :py:attr:`_pages_data`
        attribute

        :return: list of pages data
        """
        return self._pages_data

    def get_pages(self):
        """
        Create pages using self._pages_data and self.languages

        :return: list of created pages
        """
        return self.create_pages(self._pages_data, self.languages)

    @staticmethod
    def create_pages(source, languages):
        """
        Build pages according to the pages data provided by :py:meth:`get_pages_data`
        and returns the list of the draft version of each
        """
        from cms.api import create_page, create_title

        pages = OrderedDict()
        has_apphook = False
        home_set = False
        for page_data in source:
            main_data = deepcopy(page_data[languages[0]])
            if "publish" in main_data:
                main_data["published"] = main_data.pop("publish")
            main_data["language"] = languages[0]
            if main_data.get("parent", None):
                main_data["parent"] = pages[main_data["parent"]]
            page = create_page(**main_data)
            has_apphook = has_apphook or "apphook" in main_data
            for lang in languages[1:]:
                if lang in page_data:
                    publish = False
                    title_data = deepcopy(page_data[lang])
                    if "publish" in title_data:
                        publish = title_data.pop("publish")
                    if "published" in title_data:
                        publish = title_data.pop("published")
                    title_data["language"] = lang
                    title_data["page"] = page
                    create_title(**title_data)
                    if publish:
                        page.publish(lang)
            if not home_set and hasattr(page, "set_as_homepage") and main_data.get("published", False):
                page.set_as_homepage()
                home_set = True
            page = page.get_draft_object()
            pages[page.get_slug(languages[0])] = page
        if has_apphook:
            reload_urls(settings, cms_apps=True)
        return list(pages.values())

    def get_content_renderer(self, request):
        """
        Returns a the plugin renderer. Only for django CMS 3.4+

        :param request: request instance
        :return: ContentRenderer instance
        """
        from cms.plugin_rendering import ContentRenderer

        return ContentRenderer(request)

    def get_plugin_context(self, page, lang, plugin, edit=False):
        """
        Returns a context suitable for CMSPlugin.render_plugin / render_placeholder

        :param page: Page object
        :param lang: Current language
        :param plugin: Plugin instance
        :param edit: Enable edit mode for rendering
        :return: PluginContext instance
        """
        from cms.plugin_rendering import PluginContext
        from sekizai.context_processors import sekizai

        request = self.get_page_request(page, self.user, lang=lang, edit=edit)
        context = {"request": request}
        renderer = self.get_content_renderer(request)
        if renderer:
            context["cms_content_renderer"] = renderer
        context.update(sekizai(request))
        return PluginContext(context, plugin, plugin.placeholder)

    def render_plugin(self, page, lang, plugin, edit=False):
        """
        Renders a single plugin using CMSPlugin.render_plugin

        :param page: Page object
        :param lang: Current language
        :param plugin: Plugin instance
        :param edit: Enable edit mode for rendering
        :return: Rendered plugin
        """
        context = self.get_plugin_context(page, lang, plugin, edit)
        content_renderer = context["cms_content_renderer"]
        rendered = content_renderer.render_plugin(instance=plugin, context=context, placeholder=plugin.placeholder)
        return rendered

    def _prepare_request(self, request, page, user, lang, use_middlewares, use_toolbar=False, secure=False):
        from django.contrib.auth.models import AnonymousUser
        from importlib import import_module

        engine = import_module(settings.SESSION_ENGINE)

        request.current_page = SimpleLazyObject(lambda: page)
        if not user:
            if self._login_context:
                user = self._login_context.user
            else:
                user = AnonymousUser()
        if user.is_authenticated:
            session_key = user._meta.pk.value_to_string(user)
        else:
            session_key = "session_key"

        request.user = user
        request._cached_user = user
        request.session = engine.SessionStore(session_key)
        if secure:
            request.environ["SERVER_PORT"] = str("443")
            request.environ["wsgi.url_scheme"] = str("https")
        request.cookies = SimpleCookie()
        request.errors = StringIO()
        request.LANGUAGE_CODE = lang
        if request.method == "POST":
            request._dont_enforce_csrf_checks = True
        # Let's use middleware in case requested, otherwise just use CMS toolbar if needed
        if use_middlewares:
            self._apply_middlewares(request)
        elif use_toolbar:
            from cms.middleware.toolbar import ToolbarMiddleware

            mid = ToolbarMiddleware()
            mid.process_request(request)
        return request

    def _apply_middlewares(self, request):
        handler = BaseHandler()
        from django.utils.module_loading import import_string

        for middleware_path in reversed(settings.MIDDLEWARE):
            middleware = import_string(middleware_path)
            mw_instance = middleware(handler)
            if hasattr(mw_instance, "process_request"):
                mw_instance.process_request(request)

    def request(
        self,
        path,
        method="get",
        data=None,
        page=None,
        lang="",
        user=None,
        use_middlewares=False,
        secure=False,
        use_toolbar=False,
    ):
        """
        Create a request for the given parameters.

        Request will be enriched with:

        * session
        * cookies
        * user (Anonymous if :param:user is `None`)
        * django CMS toolbar (is set)
        * current_page (if provided)

        :param path: request path
        :type path: str
        :param method: HTTP verb to use
        :type method: str
        :param data: payload to pass to the underlying :py:class:`RequestFactory` method
        :type data: dict
        :param page: current page object
        :type page: cms.models.Page
        :param lang: request language
        :type lang: str
        :param user: current user
        :type user: :py:class:`django.contrib.auth.AbstractUser`
        :param use_middlewares: pass the request through configured middlewares
        :type use_middlewares: bool
        :param secure: create HTTPS request
        :type secure: bool
        :param use_toolbar: add django CMS toolbar
        :type secure: bool
        :return: request
        """
        request = getattr(RequestFactory(), method)(path, data=data, secure=secure)
        return self._prepare_request(
            request, page, user, lang, use_middlewares, secure=secure, use_toolbar=use_toolbar,
        )

    def get_request(self, page, lang, user=None, path=None, use_middlewares=False, secure=False, use_toolbar=False):
        """
        Create a GET request for the given page and language

        :param page: current page object
        :param lang: request language
        :param user: current user
        :param path: path (if different from the current page path)
        :param use_middlewares: pass the request through configured middlewares.
        :param secure: create HTTPS request
        :param use_toolbar: add django CMS toolbar
        :return: request
        """
        path = path or page and page.get_absolute_url(lang)
        return self.request(
            path,
            method="get",
            data={},
            page=page,
            lang=lang,
            user=user,
            use_middlewares=use_middlewares,
            secure=secure,
            use_toolbar=use_toolbar,
        )

    def post_request(
        self, page, lang, data, user=None, path=None, use_middlewares=False, secure=False, use_toolbar=False,
    ):
        """
        Create a POST request for the given page and language with CSRF disabled

        :param page: current page object
        :param lang: request language
        :param data: POST payload
        :param user: current user
        :param path: path (if different from the current page path)
        :param use_middlewares: pass the request through configured middlewares.
        :param secure: create HTTPS request
        :param use_toolbar: add django CMS toolbar
        :return: request
        """
        path = path or page and page.get_absolute_url(lang)
        return self.request(
            path,
            method="post",
            data=data,
            page=page,
            lang=lang,
            user=user,
            use_middlewares=use_middlewares,
            secure=secure,
            use_toolbar=use_toolbar,
        )

    def get_page_request(self, page, user, path=None, edit=False, lang="en", use_middlewares=False, secure=False):
        """
        Create a GET request for the given page suitable for use the
        django CMS toolbar

        This method requires django CMS installed to work. It will raise an ImportError otherwise;
        not a big deal as this method makes sense only in a django CMS environment

        :param page: current page object
        :param user: current user
        :param path: path (if different from the current page path)
        :param edit: whether enabling editing mode
        :param lang: request language
        :param use_middlewares: pass the request through configured middlewares.
        :param secure: create HTTPS request
        :return: request
        """
        from cms.utils.conf import get_cms_setting

        edit_on = get_cms_setting("CMS_TOOLBAR_URL__EDIT_ON")
        path = path or page and page.get_absolute_url(lang)
        if edit:
            path = "{0}?{1}".format(path, edit_on)
        request = self.request_factory.get(path, secure=secure)
        return self._prepare_request(request, page, user, lang, use_middlewares, use_toolbar=True, secure=secure)

    @staticmethod
    def create_image(mode="RGB", size=(800, 600)):
        """
        Create a random image suitable for saving as DjangoFile
        :param mode: color mode
        :param size: tuple of width, height
        :return: image object

        It requires Pillow installed in the environment to work

        """
        from PIL import Image as PilImage, ImageDraw

        image = PilImage.new(mode, size)
        draw = ImageDraw.Draw(image)
        x_bit, y_bit = size[0] // 10, size[1] // 10
        draw.rectangle((x_bit, y_bit * 2, x_bit * 7, y_bit * 3), "red")
        draw.rectangle((x_bit * 2, y_bit, x_bit * 3, y_bit * 8), "red")
        return image

    def create_django_image_obj(self):  # pragma: no cover
        return self.create_django_image_object()

    def create_django_image_object(self):
        """
        Create a django image file object suitable for FileField
        It also sets the following attributes:

        * ``self.image_name``: the image base name
        * ``self.filename``: the complete image path

        :return: django file object

        It requires Pillow installed in the environment to work

        """
        img_obj, self.filename = self.create_django_image()
        self.image_name = img_obj.name
        return img_obj

    @staticmethod
    def create_django_image():
        """
        Create a django image file object suitable for FileField
        It also sets the following attributes:

        * ``self.image_name``: the image base name
        * ``self.filename``: the complete image path

        :return: (django file object, path to file image)

        It requires Pillow installed in the environment to work
        """
        from django.core.files import File as DjangoFile

        img = BaseTestCase.create_image()
        image_name = "test_file.jpg"
        if settings.FILE_UPLOAD_TEMP_DIR:
            tmp_dir = settings.FILE_UPLOAD_TEMP_DIR
        else:
            tmp_dir = mkdtemp()
        filename = os.path.join(tmp_dir, image_name)
        img.save(filename, "JPEG")
        return DjangoFile(open(filename, "rb"), name=image_name), filename

    def create_filer_image_object(self):
        """
        Create a filer image object suitable for FilerImageField
        It also sets the following attributes:

        * ``self.image_name``: the image base name
        * ``self.filename``: the complete image path
        * ``self.filer_image``: the filer image object

        :return: filer image object

        It requires Pillow and django-filer installed in the environment to work

        """
        self.filer_image = self.create_filer_image(self.user, self.image_name)
        return self.filer_image

    @staticmethod
    def create_filer_image(user, image_name):
        """
        Create a filer image object suitable for FilerImageField
        It also sets the following attributes:

        * ``self.image_name``: the image base name
        * ``self.filename``: the complete image path
        * ``self.filer_image``: the filer image object

        :param user: image owner
        :param image_name: image name
        :return: filer image object

        It requires Pillow and django-filer installed in the environment to work

        """
        from filer.models import Image

        file_obj, filename = BaseTestCase.create_django_image()
        filer_image = Image.objects.create(owner=user, file=file_obj, original_filename=image_name)
        return filer_image

    @contextmanager
    def captured_output(self):
        """
        Context manager that patches stdout / stderr with StringIO and return the instances.
        Use it to test output

        :return: stdout, stderr wrappers
        """
        with patch("sys.stdout", new_callable=StringIO) as out:
            with patch("sys.stderr", new_callable=StringIO) as err:
                yield out, err


class BaseTestCase(BaseTestCaseMixin, TestCase):
    """
    Base class that implements :py:class:`BaseTestCaseMixin` and
    :py:class:`django.tests.TestCase`
    """


class BaseTransactionTestCase(BaseTestCaseMixin, TransactionTestCase):
    """
    Base class that implements :py:class:`BaseTestCaseMixin` and
    :py:class:`django.tests.TransactionTestCase`
    """
